' (C) Copyright 2002-2005 by Autodesk, Inc. 
'
' Permission to use, copy, modify, and distribute this software in
' object code form for any purpose and without fee is hereby granted, 
' provided that the above copyright notice appears in all copies and 
' that both that copyright notice and the limited warranty and
' restricted rights notice below appear in all supporting 
' documentation.
'
' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
' UNINTERRUPTED OR ERROR FREE.
'
' Use, duplication, or disclosure by the U.S. Government is subject to 
' restrictions set forth in FAR 52.227-19 (Commercial Computer
' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
' (Rights in Technical Data and Computer Software), as applicable.
'

Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports System.Windows

Public Class adskClass

  ' Define command 'helloworld'
  <CommandMethod("helloworld")> _
  Public Sub helloworld()

    ' get the editor object
    Dim ed As Autodesk.AutoCAD.EditorInput.Editor = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor
    ' now write to the command line
    ed.WriteMessage(vbCr + "Hello World")

  End Sub

  <CommandMethod("addAnEnt")> _
  Public Sub AddAnEnt()

    ' get the editor object so we can carry out some input
    Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

    ' first decide what type of entity we want to create
    Dim getWhichEntityOptions As PromptKeywordOptions = New PromptKeywordOptions("Which entity do you want to create? [Circle/Block] : ", "Circle Block")
    ' now input it
    Dim getWhichEntityResult As PromptResult = ed.GetKeywords(getWhichEntityOptions)
    ' if ok
    If (getWhichEntityResult.Status = PromptStatus.OK) Then

      ' test which one is required
      Select Case getWhichEntityResult.StringResult

        Case "Circle"

          ' pick the center point of the circle
          Dim getPointOptions As PromptPointOptions = New PromptPointOptions("Pick Center Point : ")
          Dim getPointResult As PromptPointResult = ed.GetPoint(getPointOptions)
          ' if ok
          If (getPointResult.Status = PromptStatus.OK) Then

            ' the get the radius
            Dim getRadiusOptions As PromptDistanceOptions = New PromptDistanceOptions("Pick Radius : ")
            ' set the start point to the center (the point we just picked)
            getRadiusOptions.BasePoint = getPointResult.Value
            ' now tell the input mechanism to actually use the basepoint!
            getRadiusOptions.UseBasePoint = True
            ' now get the radius
            Dim getRadiusResult As PromptDoubleResult = ed.GetDistance(getRadiusOptions)
            ' if all is ok
            If (getRadiusResult.Status = PromptStatus.OK) Then

              ' need to add the circle to the current space
              ' get the current working database
              Dim dwg As Database = ed.Document.Database
              ' now start a transaction
              Dim trans As Transaction = dwg.TransactionManager.StartTransaction()
              Try

                ' create a new circle
                Dim circle As New Circle(getPointResult.Value, Vector3d.ZAxis, getRadiusResult.Value)
                ' open the current space (block table record) for write
                Dim btr As BlockTableRecord = trans.GetObject(dwg.CurrentSpaceId, OpenMode.ForWrite)
                ' now the circle to the current space, model space more than likely
                btr.AppendEntity(circle)
                ' tell the transaction about the new circle so that it can autoclose it
                trans.AddNewlyCreatedDBObject(circle, True)
                ' now commit the transaction
                trans.Commit()

              Catch ex As Exception
                ' ok so we have an exception
                ed.WriteMessage("problem due to " + ex.Message)
              Finally
                ' all done, whether an error on not - dispose the transaction.
                trans.Dispose()
              End Try

            End If
          End If

        Case "Block"

          ' enter the name of the block
          Dim blockNameOptions As PromptStringOptions = New PromptStringOptions("Enter name of the Block to create : ")
          ' no spaces are allowed in a blockname so disable it
          blockNameOptions.AllowSpaces = False
          ' get the name
          Dim blockNameResult As PromptResult = ed.GetString(blockNameOptions)
          ' if ok
          If (blockNameResult.Status = PromptStatus.OK) Then

            ' lets create the block definition
            ' get the current drawing
            Dim dwg As Database = ed.Document.Database
            ' now start a transaction
            Dim trans As Transaction = dwg.TransactionManager.StartTransaction
            Try

              ' create the new block definition
              Dim newBlockDef As BlockTableRecord = New BlockTableRecord
              ' name the block definition
              newBlockDef.Name = blockNameResult.StringResult

              ' now add the new block defintion to the block table
              ' open the blok table for read so we can check to see if the name already exists
              Dim blockTable As BlockTable = trans.GetObject(dwg.BlockTableId, OpenMode.ForRead)
              ' check to see if the block already exists
              If (blockTable.Has(blockNameResult.StringResult) = False) Then
                ' if it's not there, then we are ok to add it
                ' but first we need to upgrade the open to write
                blockTable.UpgradeOpen()
                blockTable.Add(newBlockDef)
                ' tell the transaction manager about the new object so that the transaction will autoclose it
                trans.AddNewlyCreatedDBObject(newBlockDef, True)
              End If

              ' now add some objects to the block definition
              Dim circle1 As New Circle(New Point3d(0, 0, 0), Vector3d.ZAxis, 10)
              Dim circle2 As New Circle(New Point3d(20, 10, 0), Vector3d.ZAxis, 10)
              Dim circle3 As New Circle(New Point3d(40, 20, 0), Vector3d.ZAxis, 10)
              Dim circle4 As New Circle(New Point3d(60, 30, 0), Vector3d.ZAxis, 10)
              Dim circle5 As New Circle(New Point3d(80, 40, 0), Vector3d.ZAxis, 10)
              newBlockDef.AppendEntity(circle1)
              newBlockDef.AppendEntity(circle2)
              newBlockDef.AppendEntity(circle3)
              newBlockDef.AppendEntity(circle4)
              newBlockDef.AppendEntity(circle5)
              ' tell the transaction manager about the new objects so that the transaction will autoclose it
              trans.AddNewlyCreatedDBObject(circle1, True)
              trans.AddNewlyCreatedDBObject(circle2, True)
              trans.AddNewlyCreatedDBObject(circle3, True)
              trans.AddNewlyCreatedDBObject(circle4, True)
              trans.AddNewlyCreatedDBObject(circle5, True)

              ' now set where it should appear in the current space
              Dim blockRefPointOptions As PromptPointOptions = New PromptPointOptions("Pick insertion point of Block : ")
              Dim blockRefPointResult As PromptPointResult = ed.GetPoint(blockRefPointOptions)
              ' check to see if everything was ok - if not
              If (blockRefPointResult.Status <> PromptStatus.OK) Then
                ' dispose of everything that we have done so far and return
                trans.Dispose()
                Return
              End If
              ' now we have the block defintion in place and the position we need to create the reference to it
              Dim blockRef As BlockReference = New BlockReference(blockRefPointResult.Value, newBlockDef.ObjectId)
              ' otherwise add it to the current space, first open the current space for write
              Dim curSpace As BlockTableRecord = trans.GetObject(dwg.CurrentSpaceId, OpenMode.ForWrite)
              ' now add the block reference to it
              curSpace.AppendEntity(blockRef)
              ' remember to tell the transaction about the new block reference so that the transaction can autoclose it
              trans.AddNewlyCreatedDBObject(blockRef, True)

              ' all ok, commit it
              trans.Commit()

            Catch ex As Exception
              ' a problem occured, lets print it
              ed.WriteMessage("a problem occured because " + ex.Message)
            Finally
              ' whatever happens we must dispose the transaction
              trans.Dispose()

            End Try

          End If

      End Select
    End If

  End Sub

  ' declare a paletteset object, this will only be created once
  Public myPaletteSet As Autodesk.AutoCAD.Windows.PaletteSet
  ' we need a palette which will be housed by the paletteSet
  Public myPalette As UserControl1

  ' palette command
  <CommandMethod("palette")> _
  Public Sub palette()

    ' check to see if it is valid
    If (myPaletteSet = Nothing) Then
      ' create a new palette set, with a unique guid
      myPaletteSet = New Autodesk.AutoCAD.Windows.PaletteSet("My Palette", New Guid("D61D0875-A507-4b73-8B5F-9266BEACD596"))
      ' now create a palette inside, this has our tree control
      myPalette = New UserControl1
      ' now add the palette to the paletteset
      myPaletteSet.Add("Palette1", myPalette)

    End If

    ' now display the paletteset
    myPaletteSet.Visible = True

  End Sub

  <CommandMethod("addDBEvents")> _
  Public Sub addDBEvents()

    ' the palette needs to be created for this
    If myPalette Is Nothing Then
      ' get the editor object
      Dim ed As Autodesk.AutoCAD.EditorInput.Editor = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor
      ' now write to the command line
      ed.WriteMessage(vbCr + "Please call the 'palette' command first")
      Exit Sub
    End If

    ' get the current working database
    Dim curDwg As Database = Application.DocumentManager.MdiActiveDocument.Database
    ' add a handlers for what we need
    AddHandler curDwg.ObjectAppended, New ObjectEventHandler(AddressOf callback_ObjectAppended)
    AddHandler curDwg.ObjectErased, New ObjectErasedEventHandler(AddressOf callback_ObjectErased)
    AddHandler curDwg.ObjectReappended, New ObjectEventHandler(AddressOf callback_ObjectReappended)
    AddHandler curDwg.ObjectUnappended, New ObjectEventHandler(AddressOf callback_ObjectUnappended)

  End Sub

  Private Sub callback_ObjectAppended(ByVal sender As Object, ByVal e As ObjectEventArgs)

    ' add the class name of the object to the tree view
    Dim newNode As System.Windows.Forms.TreeNode = myPalette.TreeView1.Nodes.Add(e.DBObject.GetType().ToString())
    ' we need to record its id for recognition later
    newNode.Tag = e.DBObject.ObjectId.ToString()

  End Sub

  Private Sub callback_ObjectErased(ByVal sender As Object, ByVal e As ObjectErasedEventArgs)

    ' if the object was erased
    If e.Erased Then
      ' find the object in the treeview control so we can remove it
      For Each node As Forms.TreeNode In myPalette.TreeView1.Nodes
        ' is this the one we want
        If (node.Tag = e.DBObject.ObjectId.ToString) Then
          node.Remove()
          Exit For
        End If
      Next
      ' if the object was unerased
    Else
      ' add the class name of the object to the tree view
      Dim newNode As System.Windows.Forms.TreeNode = myPalette.TreeView1.Nodes.Add(e.DBObject.GetType().ToString())
      ' we need to record its id for recognition later
      newNode.Tag = e.DBObject.ObjectId.ToString()
    End If
  End Sub
  Private Sub callback_ObjectReappended(ByVal sender As Object, ByVal e As ObjectEventArgs)

    ' add the class name of the object to the tree view
    Dim newNode As Forms.TreeNode = myPalette.TreeView1.Nodes.Add(e.DBObject.GetType().ToString())
    ' we need to record its id for recognition later
    newNode.Tag = e.DBObject.ObjectId.ToString()

  End Sub

  Private Sub callback_ObjectUnappended(ByVal sender As Object, ByVal e As ObjectEventArgs)

    ' find the object in the treeview control so we can remove it
    For Each node As Forms.TreeNode In myPalette.TreeView1.Nodes
      ' is this the one we want
      If (node.Tag = e.DBObject.ObjectId.ToString) Then
        node.Remove()
        Exit For
      End If
    Next

  End Sub

  <CommandMethod("addData")> _
  Public Sub addData()

    ' get the editor object
    Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
    ' pick entity to add data to!
    Dim getEntityResult As PromptEntityResult = ed.GetEntity("Pick an entity to add an Extension Dictionary to : ")
    ' if all was ok
    If (getEntityResult.Status = PromptStatus.OK) Then
      ' now start a transaction
      Dim trans As Transaction = ed.Document.Database.TransactionManager.StartTransaction
      Try

        ' open the entity for read
        Dim ent As Entity = trans.GetObject(getEntityResult.ObjectId, OpenMode.ForRead)
        ' check to see if there is an extension dictionary available for this entity
        If (ent.ExtensionDictionary.IsNull) Then
          ' create a new one
          ' upgrade the open to write
          ent.UpgradeOpen()
          ' now create it
          ent.CreateExtensionDictionary()
        End If
        ' ok must be valid now
        ' open it for read
        Dim extensionDict As DBDictionary = trans.GetObject(ent.ExtensionDictionary, OpenMode.ForRead)

        Try
          ' check to see if our entry is in there, excpetion will be thrown if not so process that
          ' condition in the catch
          Dim entryId As ObjectId = extensionDict.GetAt("MyData")

          ' if we are here, then all is ok
          ' print the data
          ed.WriteMessage(ControlChars.Lf + "This entity already has data...")
          ' ok extract the xrecord
          Dim myXrecord As Xrecord
          ' read it from the extension dictionary
          myXrecord = trans.GetObject(entryId, OpenMode.ForRead)
          ' now print out the values
          For Each value As TypedValue In myXrecord.Data
            ed.WriteMessage(ControlChars.Lf + value.TypeCode.ToString() + " . " + value.Value.ToString())
          Next

        Catch
          ' upgrade to write status
          extensionDict.UpgradeOpen()

          ' create a new XRecord
          Dim myXrecord As New Xrecord
          ' create the resbuf list
          Dim data As ResultBuffer = New ResultBuffer(New TypedValue(DxfCode.Int16, 1), _
                                                      New TypedValue(DxfCode.Text, "MyStockData"), _
                                                    New TypedValue(DxfCode.Real, 51.9), _
                                                    New TypedValue(DxfCode.Real, 100.0), _
                                                    New TypedValue(DxfCode.Real, 320.6))
          ' now add it to the xrecord
          myXrecord.Data = data

          ' create the entry
          extensionDict.SetAt("MyData", myXrecord)
          ' tell the transaction about the newly created xrecord
          trans.AddNewlyCreatedDBObject(myXrecord, True)

          ' now tell our treeview control about the new data
          ' find the tag we want in the tree
          ' a good idea to check to see if the palette has been started, if not then it might crash
          If Not myPalette Is Nothing Then
            For Each node As Forms.TreeNode In Me.myPalette.TreeView1.Nodes
              ' if it's the one we want
              If (node.Tag = ent.ObjectId.ToString()) Then
                ' add the new data to the treenode
                Dim childNode As Forms.TreeNode = node.Nodes.Add("Extension Dictionary")
                ' now add the data
                For Each value As TypedValue In myXrecord.Data
                  childNode.Nodes.Add(value.ToString())
                Next

                ' all done - break out of the loop
                Exit For
              End If
            Next
          End If
        End Try

        ' all ok, commit it
        trans.Commit()

      Catch ex As Exception
        ' a problem occured, lets print it
        ed.WriteMessage("a problem occured because " + ex.Message)
      Finally
        ' whatever happens we must dispose the transaction
        trans.Dispose()

      End Try

    End If

  End Sub

  <CommandMethod("addDataToNOD")> _
    Public Sub addDataToNOD()

    ' get the editor object
    Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
    ' pick entity to add data to!
    Dim trans As Transaction = ed.Document.Database.TransactionManager.StartTransaction
    Try

      ' open the NOD for read
      Dim nod As DBDictionary = trans.GetObject(ed.Document.Database.NamedObjectsDictionaryId, OpenMode.ForRead)

      Try
        ' check to see if our entry is in there, excpetion will be thrown if not so process that
        ' condition in the catch
        Dim entryId As ObjectId = nod.GetAt("MyData")

        ' if we are here, then all is ok
        ' print the data
        ed.WriteMessage(ControlChars.Lf + "This entity already has data...")
        ' ok extract the xrecord
        Dim myXrecord As Xrecord
        ' read it from the NOD dictionary
        myXrecord = trans.GetObject(entryId, OpenMode.ForRead)
        ' now print out the values
        For Each value As TypedValue In myXrecord.Data
          ed.WriteMessage(ControlChars.Lf + value.TypeCode.ToString() + " . " + value.Value.ToString())
        Next

      Catch
        ' upgrade to write status
        nod.UpgradeOpen()

        ' create a new XRecord
        Dim myXrecord As New Xrecord
        ' create the resbuf list
        Dim data As ResultBuffer = New ResultBuffer(New TypedValue(DxfCode.Int16, 1), _
                                                    New TypedValue(DxfCode.Text, "MyCompanyDefaultSettings"), _
                                                    New TypedValue(DxfCode.Real, 51.9), _
                                                    New TypedValue(DxfCode.Real, 100.0), _
                                                    New TypedValue(DxfCode.Real, 320.6))
        ' now add it to the xrecord
        myXrecord.Data = data

        ' create the entry
        nod.SetAt("MyData", myXrecord)
        ' tell the transaction about the newly created xrecord
        trans.AddNewlyCreatedDBObject(myXrecord, True)

      End Try

      ' all ok, commit it
      trans.Commit()

    Catch ex As Exception
      ' a problem occurred, lets print it
      ed.WriteMessage("a problem occurred because " + ex.Message)
    Finally
      ' whatever happens we must dispose the transaction
      trans.Dispose()

    End Try

  End Sub

  <CommandMethod("pointmonitor")> _
  Public Sub startMonitor()

    ' get the editor object
    Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
    ' now add the delegate to the events list
    AddHandler ed.PointMonitor, New PointMonitorEventHandler(AddressOf MyPointMonitor)

  End Sub

  Public Sub MyPointMonitor(ByVal sender As Object, ByVal e As PointMonitorEventArgs)

    ' first lets check what is under the Cursor
    Dim fullEntPath() As FullSubentityPath = e.Context.GetPickedEntities()
    If (fullEntPath.Length) Then
      ' ok first lets display a ToolTip
      ' start a transaction
      Dim trans As Transaction = Application.DocumentManager.MdiActiveDocument.Database.TransactionManager.StartTransaction
      Try

        ' open the Entity for read
        Dim ent As Entity = trans.GetObject(fullEntPath(0).GetObjectIds(0), OpenMode.ForRead)
        e.AppendToolTipText("The Entity is a " + ent.GetType().ToString())

        ' ok, so if we are over something - then lets highlight it in the palette Window
        ' find the tag we want in the tree
        Dim fontRegular As System.Drawing.Font = New System.Drawing.Font("Microsoft Sans Serif", 8, Drawing.FontStyle.Regular)
        Dim fontBold As System.Drawing.Font = New System.Drawing.Font("Microsoft Sans Serif", 8, Drawing.FontStyle.Bold)

        ' wait until after we have processed the whole tree before we handle the different sized text
        Me.myPalette.TreeView1.SuspendLayout()

        For Each node As Forms.TreeNode In Me.myPalette.TreeView1.Nodes
          ' if it's the one we want
          If (node.Tag = ent.ObjectId.ToString()) Then
            If Not fontBold.Equals(node.NodeFont) Then
              node.NodeFont = fontBold
              ' 
              node.Text = node.Text
            End If
          Else
            If Not fontRegular.Equals(node.NodeFont) Then
              node.NodeFont = fontRegular
            End If
          End If
        Next

        ' now it's time to recalc the layout of the treeview
        Me.myPalette.TreeView1.ResumeLayout()
        Me.myPalette.TreeView1.Refresh()
        Me.myPalette.TreeView1.Update()

        ' all ok, commit it
        trans.Commit()
      Catch ex As Exception

      Finally
        ' whatever happens we must dispose the transaction
        trans.Dispose()

      End Try

    End If

    ' ok, so now lets do something really pointless! space invaders!
    If (e.Context.History And PointHistoryBits.NotDigitizer) Then
      ' this means the mouse has been clicked... so...
      ' create a Circle
      Dim circle As Circle = New Circle(e.Context.RawPoint, Vector3d.ZAxis, 10)
      ' draw it
      e.Context.DrawContext.Geometry.Draw(circle)
      ' wait for a while
      System.Threading.Thread.Sleep(50)
      ' draw it
      circle.Radius = 20
      e.Context.DrawContext.Geometry.Draw(circle)
      ' wait for a while
      System.Threading.Thread.Sleep(50)
      ' draw it
      circle.Radius = 40
      e.Context.DrawContext.Geometry.Draw(circle)
      ' wait for a while
      System.Threading.Thread.Sleep(50)
      ' draw it
      circle.Radius = 80
      e.Context.DrawContext.Geometry.Draw(circle)
      ' wait for a while
      System.Threading.Thread.Sleep(50)

    End If

  End Sub

  <CommandMethod("newInput")> _
  Public Sub NewInput()

    ' start our input point Monitor    
    ' get the editor object
    Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
    ' now add the delegate to the events list
    AddHandler ed.PointMonitor, New PointMonitorEventHandler(AddressOf MyInputMonitor)
    ed.TurnForcedPickOn()

    ' now pick some point, this emulates a real world input that may be required
    ' pick the center point of the circle
    Dim getPointOptions As PromptPointOptions = New PromptPointOptions("Pick A Point : ")
    Dim getPointResult As PromptPointResult = ed.GetPoint(getPointOptions)
    ' if ok
    If (getPointResult.Status = PromptStatus.OK) Then
      ' do something...
    End If

    ' now remove our point monitor as we are finished With it
    RemoveHandler ed.PointMonitor, AddressOf MyInputMonitor

  End Sub

  Public Sub MyInputMonitor(ByVal sender As Object, ByVal e As PointMonitorEventArgs)

    ' first lets check what is under the Cursor
    Dim fullEntPath() As FullSubentityPath = e.Context.GetPickedEntities()
    If (fullEntPath.Length) Then

      ' start a transaction
      Dim trans As Transaction = Application.DocumentManager.MdiActiveDocument.Database.TransactionManager.StartTransaction
      Try

        ' open the Entity for read, it must be derived from Curve
        Dim ent As Curve = trans.GetObject(fullEntPath(0).GetObjectIds(0), OpenMode.ForRead)

        ' ok, so if we are over something - then check to see if it has an extension dictionary
        If (ent.ExtensionDictionary.IsValid) Then

          ' open it for read
          Dim extensionDict As DBDictionary = trans.GetObject(ent.ExtensionDictionary, OpenMode.ForRead)

          ' find the entry
          Dim entryId As ObjectId = extensionDict.GetAt("MyData")

          ' if we are here, then all is ok
          ' extract the xrecord
          Dim myXrecord As Xrecord
          ' read it from the extension dictionary
          myXrecord = trans.GetObject(entryId, OpenMode.ForRead)
          ' now draw the positions 
          For Each value As TypedValue In myXrecord.Data
            ' if the TypeCode is a distance value
            If (value.TypeCode = DxfCode.Real) Then
              ' draw some temporary graphics along the Curve to show the distances

              ' first get the point along the curve
              Dim pnt As Point3d = ent.GetPointAtDist(value.Value)

              ' next work out how many pixels are in a unit square - so we can keep the temporary graphics a set size
              Dim pixels As Point2d = e.Context.DrawContext.Viewport.GetNumPixelsInUnitSquare(pnt)
              ' setup the constant distances
              Dim xDist As Double = 10 / pixels.X
              Dim yDist As Double = 10 / pixels.Y

              ' then draw the temporary Graphics
              Dim circle As Circle = New Circle(pnt, Vector3d.ZAxis, xDist)
              e.Context.DrawContext.Geometry.Draw(circle)

              ' more temporary graphics
              Dim text As DBText = New DBText
              ' always a good idea to set the Database defaults With things like text, dimensions etc
              text.SetDatabaseDefaults()
              ' set the position of the text to the same point as the circle, but offset by the radius
              text.Position = pnt + New Vector3d(xDist, yDist, 0)
              text.TextString = value.Value.ToString()
              text.Height = yDist
              e.Context.DrawContext.Geometry.Draw(text)
            End If
          Next
        End If

        ' all ok, commit it
        trans.Commit()
      Catch ex As Exception

      Finally
        ' whatever happens we must dispose the transaction
        trans.Dispose()

      End Try

    End If

  End Sub

  ' entity jig, handle one entity at a time
  Class MyCircleJig
    Inherits EntityJig

    ' we need member variable for each parameter that we are going to jig
    Private centerPoint As Point3d
    Private radius As Double
    ' because we are going to have 2 inputs, a center point, then a radius we need to keep track of the input number
    Private currentInputValue As Integer
    ' this property will be accessed outside of the class, so lets create a property get and set for it
    Property CurrentInput() As Integer
      Get
        Return currentInputValue
      End Get
      Set(ByVal value As Integer)
        currentInputValue = value
      End Set


    End Property

    ' create the default constructor
    Sub New(ByVal ent As Entity)

      ' MyBase.New must be called because base class 'EntityJig' does not have an accessible 'Sub New' that can be called with no arguments
      MyBase.New(ent)

    End Sub


    ' this function is called to "sample" the input, we decide how to do that
    Protected Overrides Function Sampler(ByVal prompts As Autodesk.AutoCAD.EditorInput.JigPrompts) As Autodesk.AutoCAD.EditorInput.SamplerStatus

      Select Case (currentInputValue)

        Case 0
          ' remember the original point
          Dim oldPnt As Point3d = centerPoint
          ' now pick the center point of the circle
          Dim jigPromptResult As PromptPointResult = prompts.AcquirePoint("Pick center point : ")
          ' now check the status
          If (jigPromptResult.Status = PromptStatus.OK) Then

            ' extract out the point from the result
            centerPoint = jigPromptResult.Value

            ' check to see if the user made a change
            If (oldPnt.DistanceTo(centerPoint) < 0.001) Then
              Return SamplerStatus.NoChange
            End If
          End If

          ' otherwise, return the status
          Return SamplerStatus.OK

        Case 1

          ' remember the old radius
          Dim oldRadius As Double = radius
          ' now jig the radius distance - first setup the input requirements, start point etc
          Dim jigPromptDistanceOpts As New JigPromptDistanceOptions("Pick radius : ")
          jigPromptDistanceOpts.UseBasePoint = True
          jigPromptDistanceOpts.BasePoint = centerPoint
          ' ok now ready to input
          Dim jigPromptResult As PromptDoubleResult = prompts.AcquireDistance(jigPromptDistanceOpts)
          ' same as before, check the return status
          If (jigPromptResult.Status = PromptStatus.OK) Then

            ' extract out the point from the result
            radius = jigPromptResult.Value

            ' check to see if the radius is too small
            If (System.Math.Abs(radius) < 0.1) Then
              radius = 1
            End If

            ' check to see if the user made a change
            If (System.Math.Abs(oldRadius - radius) < 0.001) Then
              Return SamplerStatus.NoChange
            End If
          End If

          ' otherwise, return the status
          Return SamplerStatus.OK

      End Select


    End Function

    ' for every input, we need to update the entity
    Protected Overrides Function Update() As Boolean

      ' see which input we are on
      Select Case (currentInputValue)

        Case 0
          ' the jig stores the circle as an Entity type - change it to a circle so we can access the properties easily
          CType(Me.Entity, Circle).Center = centerPoint

        Case 1
          ' the jig stores the circle as an Entity type - change it to a circle so we can access the properties easily
          CType(Me.Entity, Circle).Radius = radius

      End Select

    End Function
  End Class

  ' create a command to invoke the EntityJig
  <CommandMethod("circleJig")> _
  Public Sub CircleJig()

    ' create a new instance of the circle we want to form with the jig
    Dim circle As Circle = New Circle(Point3d.Origin, Vector3d.ZAxis, 10)
    ' create a new jig
    Dim jig As New MyCircleJig(circle)

    ' now loop the inputs, 0 will be to position the circle and 1 will be to set the radius
    For i As Integer = 0 To 2

      ' set the current input!
      jig.CurrentInput = i

      ' get the editor object
      Dim ed As Autodesk.AutoCAD.EditorInput.Editor = Application.DocumentManager.MdiActiveDocument.Editor
      ' now invoke the jig
      Dim promptResult As PromptResult = ed.Drag(jig)
      ' all ok
      If (promptResult.Status = PromptStatus.Cancel Or promptResult.Status = PromptStatus.Error) Then
        Return
      End If

    Next

    ' once we are finished with the jig, time to add the newly formed circle to the database
    ' get the working database
    Dim dwg As Database = Application.DocumentManager.MdiActiveDocument.Database
    ' now start a transaction
    Dim trans As Transaction = dwg.TransactionManager.StartTransaction
    Try

      ' open the current space for write
      Dim currentSpace As BlockTableRecord = trans.GetObject(dwg.CurrentSpaceId, OpenMode.ForWrite)
      ' add it to the current space
      currentSpace.AppendEntity(circle)
      ' tell the transaction manager about it
      trans.AddNewlyCreatedDBObject(circle, True)

      ' all ok, commit it
      trans.Commit()

    Catch ex As Exception
    Finally
      ' whatever happens we must dispose the transaction
      trans.Dispose()

    End Try

  End Sub
End Class
